#include "App.h"
#include "PINS.h"
#include "IMU.h"
#include "LCD.h"
#include "BAND.h"
#include "FONT.h"
#include "GRAPH.h"
#include "MENU.h"
#include "RTC.h"
#include "CFG.h"
#include "Printer.h"

App app;

void App::Init()
{
  pinMode(PIN_LED, OUTPUT);
  digitalWrite(PIN_LED, LOW);
  
  btn1Green.Init(PIN_BTN_GREEN);
  btn2Red.Init(PIN_BTN_RED);

  imu.Init(true, true);
  lcd.Init(PIN_LATCH, PIN_CLOCK, PIN_DATA);
  graph.Init();
  printer.Init();
  rtc.Init();
  if (btn2Red.IsDown())  // reset if red down on power up
  {
    cfg.Defaults();
  }
  else
  {
    // splash           1234567890123456
    lcd.Clear();
    lcd.BigFont(1, 1,   "Sleepy-Timer"); 
    lcd.BigFont(2, 0, "Mark Wilson 2012");
    lcd.InvertLine(1);
    delay(3000);
    cfg.Defaults();
    cfg.Load();
  }
  
  menu.Init();
  data.Init();
}

void App::Run()
{
  do 
  {
    Menu::tMenuItems Item = menu.DoMainMenu();
    if (Item == Menu::eStartMenu)
    {
      data.Start();
    }
    else if (Item == Menu::eGREEN_RED)
    {
      // Red, then Green (i.e. both at once)->Clock (until another button)
      doClock();
    }
  } while (true);
}

////////////////////
word p4x4Font[] =   // really 4th column is the inter-char gap
{
  0x7557,
  0x6227,
  0x6127,
  0x7317,
  0x4571,

  0x7616,
  0x4757,
  0x7111,
  0x7757,
  0x7571,
};

void App::doClock()
{
  // the obligatory clock...
  byte DisplayedMinute = 255;
  lcd.Clear();
  lcd.ClearGraphics();
  lcd.ShowGraphics(false);

  lcd.StartCustomChar(1);
  lcd.Data(0);lcd.Data(0);
  for (int Idx = 0; Idx < 14; Idx++)
  {
    lcd.Data(B00000000);lcd.Data(B01111110);
  }
  lcd.Data(0);lcd.Data(0);
  
  lcd.StartCustomChar(2);
  lcd.Data(0);lcd.Data(0);
  for (int Idx = 0; Idx < 14; Idx++)
  {
    lcd.Data(B01111110);lcd.Data(B00000000);
  }
  lcd.Data(0);lcd.Data(0);
  
  lcd.StartCustomChar(3);
  lcd.Data(0);lcd.Data(0);
  for (int Idx = 0; Idx < 14; Idx++)
  {
    lcd.Data(B01111110);lcd.Data(B01111110);
  }
  lcd.Data(0);lcd.Data(0);

  do
  {
    byte ThisMinute = rtc.ReadMinute();
    if (ThisMinute != DisplayedMinute)
    {
      char Buffer[6];
      Buffer[0] = 0;
      rtc.ReadTime();
      DisplayedMinute = rtc.m_Minute;
      GRAPH::AppendNum(rtc.m_Hour24, Buffer, '0');
      GRAPH::AppendNum(rtc.m_Minute, Buffer, '0');
      for (int Row = 0; Row < LCD_CHAR_ROWS; Row++)
      {
        char* pCh = Buffer;
        int Col = 0;
        while (*pCh)
        {
          word Font = p4x4Font[*pCh - '0'];
          int Bit = 15 - (Row*4);
          if ((pCh - Buffer) < 2)
          {
            // push the hours one to the left, bigger gap between HH & MM
            Font <<= 1;
            bitWrite(Font, Bit - 4, 0);
          }
          for (int Part = 0; Part < 2; Part++)
          {
            word DoubleChar;
            if (bitRead(Font, Bit--))
              if (bitRead(Font, Bit--))
                DoubleChar = 0x0600;  // little endian! 0 first
              else
                DoubleChar = 0x0400;
            else
              if (bitRead(Font, Bit--))
                DoubleChar = 0x0200;
              else
                DoubleChar = 0x2020;
            lcd.BigFont(Row, Col, (char*)&DoubleChar, 2);
            Col++;
          }
          pCh++;
        }
      }
    }
  } while (!btn1Green.CheckButtonPress() && !btn2Red.CheckButtonPress());
  lcd.Clear();
}  

