#include <Arduino.h>
#include "BAND.h"

tByteBand band;

byte* tByteBand::Data()
{
  // the data for sending to a printer or graphics LCD
  return m_pOutputBand;
}

byte tByteBand::Band()
{
  return m_iBandNumber;
}


void tByteBand::Init(byte Number, tBandLayout Layout)
{
  // set the layout and zero the data
  m_iLayout = Layout;
  memset(m_pOutputBand, 0, sizeof(m_pOutputBand));
  m_iBandNumber = Number;
}

void tByteBand::Fill(byte Value)
{
  memset(m_pOutputBand, Value, sizeof(m_pOutputBand));
}

void tByteBand::Dump()
{
  // dump to trace
  //for (int Idx = 0; Idx < kByteBandWidth; Idx++)
  //  TRACE("%02X ", m_pOutputBand[Idx]);
  //TRACE("\n");
  if (m_iLayout == eHorizontalBytes)
  {
    int Index = 0;
    for (int Row = 0; Row < kByteBandHeight; Row++)
    {
      byte Mask = 0x80;
      for (int Col = 0; Col < kByteBandWidth; Col++)
      {
        if (m_pOutputBand[Index] & Mask)
          Serial.print("X");
        else
          Serial.print(".");
        Mask >>= 1;
        if (!Mask)
        {
          Mask = 0x80;
          Index++;
        }
      }
      Serial.println();
    }
  }
  else
  {
    byte Mask = 0x80;
    for (int Row = 0; Row < kByteBandHeight; Row++)
    {
      for (int Col = 0; Col < kByteBandWidth; Col++)
      {
        if (m_pOutputBand[Col] & Mask)
          Serial.print("X");
        else
          Serial.print(".");
      }
      Mask >>= 1;
      Serial.println();
    }
  }
}

bool tByteBand::GetPixel(int Row, int Col, tPixelPos& Pos)
{
  // set the pixel's mask and index
  // false if outside the band
  if (0 <= Row && Row < kByteBandHeight && 0 <= Col && Col < kByteBandWidth)
  {
    Pos.m_iRow = Row;
    Pos.m_iCol = Col;

    if (m_iLayout == eHorizontalBytes)
    {
      // MSB is to the left
      // b7|b6|...|b0||b7|b6|...|b0
      // b7|b6|...|b0||b7|b6|...|b0
      // ..
      // b7|b6|...|b0||b7|b6|...|b0
      Pos.m_iMask = 0x01 << (kByteBandHeight - 1 - (Col % kByteBandHeight)); 
      Pos.m_iIndex = (Col / kByteBandHeight) + (kByteBandHeight - 1 - Row)*kByteBandWidth/kByteBandHeight;
    }
    else
    {
      // MSB is at the top
      // b7||b7||  
      // b6||b6||
      // ...
      // b0||b0||
      Pos.m_iMask = 0x01 << Row;
      Pos.m_iIndex = Col;
    }
    return true;
  }
  return false;
}

bool tByteBand::NextPixelRight(tPixelPos& Pos)
{
  // advance to the next col
  // false went outside the band
  // faster than recalculating from row/col?
  Pos.m_iCol++;
  if (m_iLayout == eHorizontalBytes)
  {
    Pos.m_iMask >>= 1;
    if (Pos.m_iMask)
    {
      return true;
    }
    else
    {
      Pos.m_iMask = 0x80;
    }
    Pos.m_iIndex++;
    return (Pos.m_iIndex % (kByteBandWidth/kByteBandHeight)) != 0;
  }
  else
  {
    Pos.m_iIndex++;
    return Pos.m_iIndex < kByteBandWidth;
  }
}

bool tByteBand::NextPixelDown(tPixelPos& Pos)
{
  // drop down a row
  // false went outside the band
  Pos.m_iRow--;
  if (m_iLayout == eHorizontalBytes)
  {
    Pos.m_iIndex += (kByteBandWidth/kByteBandHeight);
    return Pos.m_iIndex < kByteBandWidth;
  }
  else
  {
    Pos.m_iMask >>= 1;
    return Pos.m_iMask != 0;
  }
}

void tByteBand::SetPixel(const tPixelPos& Pos, bool Set)
{
  // set/clear the given pixel
  if (0 <= Pos.m_iIndex && Pos.m_iIndex < kByteBandWidth)
  {
    if (Set)
    {
      m_pOutputBand[Pos.m_iIndex] |= Pos.m_iMask;
    }
    else
    {
      m_pOutputBand[Pos.m_iIndex] &= ~Pos.m_iMask;
    }
  }
}

void tByteBand::SetPixel(int Row, int Col, bool Set)
{
  // set/clear the given pixel
  tPixelPos Pos;
  if (GetPixel(Row, Col, Pos))
  {
    SetPixel(Pos, Set);
  }
}


void tByteBand::InvertPixel(const tPixelPos& Pos)
{
  // invert the given pixel
  if (0 <= Pos.m_iIndex && Pos.m_iIndex < kByteBandWidth)
  {
    m_pOutputBand[Pos.m_iIndex] ^= Pos.m_iMask;
  }
}

bool tByteBand::IsPixelSet(tPixelPos& Pos)
{
  return (m_pOutputBand[Pos.m_iIndex] & Pos.m_iMask) != 0;
}

