#include <Arduino.h>
#include "Button.h"

#define HOLD_TIME_MS 50
#define CLOSED_STATE LOW
#define OPEN_STATE   HIGH

Button btn1Green;
Button btn2Red;

void Button::Init(int Pin)
{
  m_iPin = Pin;
  m_iPrevReading = OPEN_STATE;
  m_iPrevState = CLOSED_STATE;
  m_iTransitionTimeMS = millis();
  pinMode(m_iPin, INPUT);
  // see http://miscsolutions.wordpress.com/2011/10/16/five-things-i-never-use-in-arduino-projects/
  // "Microcontrollers have built-in switchable pullup resistors precisely so that you don’t need external resistors. 
  //  To connect a switch to an Arduino, connect it between the input pin and ground, and then in setup() enable the 
  //  internal pullup resistor by using digitalWrite() to write HIGH to the pin (that’s right, you do a write to an 
  //  input pin – that’s what you need to do to enable an internal pullup resistor on the Arduino).
  //
  //  When might I use external pullup/pulldown resistors? Only when the internal 20k pullup resistor isn’t suitable, 
  //  for example, if the switches are connected with long wires that pick up noise."
  digitalWrite(m_iPin, HIGH);
}

bool Button::CheckButtonPress()
{
  // debounced button, true if button pressed
  int ThisReading = digitalRead(m_iPin);
  if (ThisReading != m_iPrevReading)
  {
    // state change, reset the timer
    m_iPrevReading = ThisReading;
    m_iTransitionTimeMS = millis();
  }
  else if (ThisReading != m_iPrevState &&
           (millis() - m_iTransitionTimeMS) >= HOLD_TIME_MS)
  {
    // a state other than the last one and held for long enough
    m_iPrevState = ThisReading;
    return (ThisReading == CLOSED_STATE);
  }
  return false;
}

bool Button::IsDown()
{
  return digitalRead(m_iPin) == CLOSED_STATE;
}

