#ifndef _DATA_H
#define _DATA_H

#define DATA_SAMPLE_MINUTES 6
#define DATA_MAX_SAMPLES    120  // 12 hrs
#define DATA_NUM_PAGES      8    // 1k FLASH /128 bytes

class tRollingAverage
{
public:
  tRollingAverage();
  void Reset();
  int DataPoint(int Data);

private:
  enum {kSize = 8};
  int m_pDataPoints[kSize];
  int m_iDataIndex;
  int m_iSum;
  bool m_bFirst;
};

///////////////
class DATA
{
  public:
    struct tDayData  // 128 bytes
    {
      byte m_iBaseReadingBy4;
      byte m_iThresholdPercent;
      byte m_iStartDate;
      byte m_iStartMonth;

      byte m_iStartDayOfWeek;
      byte m_iStartHour24;
      byte m_iStartMinute;
      byte m_iSampleCount;
   
      // each byte represents 6 minutes, 0..100%
      byte m_pActivity[DATA_MAX_SAMPLES];
    };
    
    void Init();
    bool CheckAutoStart();
    void Start();
    
    bool LoadLast();
    bool LoadPrev();
    void GraphData();
    void PrintData();
    void EndPrint();
    void ViewTest();
    void DumpCurrentPage();
    void DumpPages();
    
  private:
    void StartLogging();
    void StopLogging();
    bool TakeReading();
    byte CalcDataValue();
    bool CheckAutoEnd(byte Minute);
    void ResetEEPROM();
    void WriteData();
    bool ReadData(int Day);
    int GetLatestDayPage();
    void CreateText(char* Line1, char* Line2);
    void GetStats(int& TotalHours, int& TotalMin, int& MaxHour, int& MaxMin);
    bool PageIsToday(int Page);
    void AppendCentiG(char* Buffer, int CentiG);
    bool IsValid(byte& Base, byte& Threshold);
  
    int m_iPageIndex;
    unsigned long m_iLastReadingMS;
    byte m_iLastReadingMinute;
    byte m_iReadingMinutes;
    tDayData m_CurrentData;
    
    tRollingAverage m_AvgX;
    tRollingAverage m_AvgY;
    tRollingAverage m_AvgZ;
    unsigned long m_iSamplePercentSum;
    int m_iSamplePercentCount;
    int m_iMaxInactivity;
    unsigned long m_iInactivityWindowSum;
    int m_iInactivityWindowCount;
    
};    

extern DATA data;
#endif

