#include <Arduino.h>
#include "FONT.h"

// ==================================================================
// Define a simple 4x5 font

// bits 7-0 hold the character, bits 27-8 hold the definition, bit 27 is the top-left of the 4-column, 5-row grid, bit 28 is a flag to indicate drop 1 row
// |<-bit 31                      bit 0->|
// 000D abcd efgh ijkl mnop qrst CCCC CCCC
// glyph:
//  abcd
//  efgh
//  ijkl
//  mnop
//  qrst
// CCCCCCCC = char
// D = descend
prog_uint32_t FONT_Defs[] PROGMEM =
{
  ' '+0x00000000, '!'+0x04440400, '%'+0x08925900, '\''+0x04400000, '('+0x02444200, ')'+0x02111200, ','+0x00022400, '-'+0x00070000, '/'+0x00124800, '0'+0x07555700, '1'+0x06222700, '2'+0x07174700, '3'+0x07171700, '4'+0x05571100, '5'+0x07471700, '6'+0x07475700, 
  '7'+0x07111100, '8'+0x06969600, '9'+0x07571700, ':'+0x00202000, ';'+0x00202400, '?'+0x07130200, 
  'A'+0x069F9900, 'B'+0x0E9E9E00, 'C'+0x07888700, 'D'+0x0E999E00, 'E'+0x0F8E8F00, 'F'+0x0F8E8800, 'G'+0x078B9700, 'H'+0x099F9900, 'I'+0x07222700, 'J'+0x07222400, 'K'+0x09ACA900, 'L'+0x08888F00, 'M'+0x09F99900, 'N'+0x09DB9900, 'O'+0x06999600, 'P'+0x0E9E8800, 'Q'+0x0699B700, 
  'R'+0x0E9EA900, 'S'+0x07861E00, 'T'+0x07222200, 'U'+0x09999600, 'V'+0x09996600, 'W'+0x0999F900, 'X'+0x09969900, 'Y'+0x05522200, 'Z'+0x0F124F00, 
  'a'+0x00079700, 'b'+0x088E9E00, 'c'+0x00078700, 'd'+0x01179700, 'e'+0x0006A700, 'f'+0x01272200, 'g'+0x10771600, 'h'+0x088E9900, 'i'+0x02022200, 'j'+0x12022400, 'k'+0x04456500, 'l'+0x02222200, 'm'+0x000FF900, 'n'+0x000E9900, 'o'+0x00069600, 'p'+0x10656400, 'q'+0x106A6300, 
  'r'+0x000E9800, 's'+0x00074C00, 't'+0x02272100, 'u'+0x00099700, 'v'+0x00095200, 'w'+0x0009FF00, 'x'+0x00052500, 'y'+0x10571600, 'z'+0x000F6F00, 
  '@'+0x06BB8700,
  0xC1+0x0FFFFF00, 
  0x00000000 // terminates table
};
#define FONT_CURSOR_CHAR '\xC1'

void FONT::SetChar(tByteBand& Band, int StartRow, int StartCol, char Char)
{
  unsigned long Defn = FindDefn(Char);
  // draw the char
  unsigned long Mask = 0x10000000;
  int YOffs = (Defn & 0x10000000)?1:0;  // drop the char
  for (int Y = 0; Y < FONT_HEIGHT; Y++)
    for (int X = 0; X < FONT_WIDTH; X++)
      Band.SetPixel(StartRow + FONT_HEIGHT - Y + YOffs, StartCol + X, (Defn & (Mask >>= 1)) != 0);
}

bool FONT::GetPixel(char Char, int Row, int Col)
{
  unsigned long Mask = 0x08000000 >> (Row*FONT_WIDTH + Col);
  unsigned long Defn = FindDefn(Char) & 0x0FFFFF00;
  return (Defn & Mask) != 0;
}

unsigned long FONT::FindDefn(char Char)
{
  int Idx = 0;
  unsigned long Defn = 0;
  // look for the char (a char not found will show as blank)
  do
  {
    Defn = pgm_read_dword_near(FONT_Defs + Idx);
    Idx++;
  } while ((unsigned char)(Char) != (Defn & 0xFF) && Defn != 0);
  return Defn;
}

void FONT::SmallFont(tByteBand& Band, int Row, int Col, char* pStr)
{
  while (*pStr)
  {
    SetChar(Band, Row, Col, *pStr);
    Col += (FONT_WIDTH + 1);
    pStr++;
  }
}
