#ifndef _LCD_H
#define _LCD_H

#include "PINS.h"

#define LCD_CLEAR  B00000001

#define LCD_HOME   B00000010

#define LCD_ENTRY  B00000100
#define LCD_ENTRY_CURSOR_RIGHT B00000010
#define LCD_ENTRY_CURSOR_LEFT  B00000000
#define LCD_ENTRY_SHIFT        B00000001

#define LCD_DISPLAY B00001000
#define LCD_DISPLAY_ON         B00000100
#define LCD_DISPLAY_OFF        B00000000
#define LCD_DISPLAY_CURSOR_ON  B00000010
#define LCD_DISPLAY_CURSOR_OFF B00000000
#define LCD_DISPLAY_BLINK_ON   B00000001
#define LCD_DISPLAY_BLINK_OFF  B00000000

#define LCD_CURSOR  B00010000

#define LCD_FUNCTION B00100000
#define LCD_FUNCTION_8BIT      B00010000
#define LCD_FUNCTION_4BIT      B00000000
#define LCD_FUNCTION_EXTENDED  B00000100
#define LCD_FUNCTION_BASIC     B00000000

#define LCD_CGRAM_ADDR B01000000

#define LCD_EXT_FUNCTION B00100000
#define LCD_EXT_FUNC_8BIT      B00010000
#define LCD_EXT_FUNC_4BIT      B00000000
#define LCD_EXT_FUNC_EXTENDED  B00000100
#define LCD_EXT_FUNC_BASIC     B00000000
#define LCD_EXT_FUNC_GRAPH_ON  B00000010
#define LCD_EXT_FUNC_GRAPH_OFF B00000000

#define LCD_EXT_SCROOL_OR_RAM  B00000010

#define LCD_INVERT     B00000100

#define LCD_GDRAM_ADDR B10000000


#define LCD_CHAR_COLUMNS 16
#define LCD_CHAR_ROWS    4

class LCD12864
{
  // 4-bit parallel interface via 74HC595 to a QC12864 128x64 LCD
  public:
    LCD12864();
    void Init(byte LatchPin, byte ClockPin, byte DataPin);
    void Clear();
    void BigFont(byte Row, byte Col, char* Str, int Len = -1);
    void InvertLine(byte Row);
    void ExtendedMode(bool Extended);
    void ShowGraphics(bool Show);
    void SetGraphicsAddress(byte Row, byte Col);
    void ShowBand(byte TopRow, byte* pData);
    void ClearGraphics(byte StartRow = 0, byte EndRow = 63);
    void StartCustomChar(byte Ch);
    void Command(byte Cmd);
    void Data(byte Dat);
    
    void BacklightOn(bool On);
    bool BacklightIsOn();
    
  private:
    enum tOutputBits  // outputs on the 74HC595
    {
      eUnused_Bit,
      
      eBacklight_Bit,
      
      eD7_Bit,  // reversed beacuse that's how they line up
      eD6_Bit,
      eD5_Bit,
      eD4_Bit,

      eEN_Bit,
      eRS_Bit,
    };

    void GetGraphicsAddress(byte Row, byte Col, byte& AddrA, byte& AddrB);
    void ShiftOut(byte Byte);
    void SendByte(byte RS, byte Byte);

    byte m_iLatchPin;
    byte m_iClockPin;
    byte m_iDataPin;
    bool m_bBacklightOn;
};

extern LCD12864 lcd;
#endif
