#ifndef _PINS_H
#define _PINS_H

#define PIN_LED 13

#define PIN_SCL    A5
#define PIN_SDA    A4

#define PIN_IMU_X  A2
#define PIN_IMU_Y  A1
#define PIN_IMU_Z  A0

// 74HC595->QC12864
#define PIN_DATA   7
#define PIN_LATCH  6
#define PIN_CLOCK  5

#define PIN_BTN_GREEN 10
#define PIN_BTN_RED    8

#define PIN_PRT_TX    9
/*
SleepyTimer -- an IMU-based sleep monitor
Mark Wilson May 2011

The Arduino is connected to 5 daughter boards:

--- CLOCK
                   +------------+
[5V]---------------+ DS1307 RTC |
[A4]---------------+-SDA        |
[A5]---------------+-SCL        |
[GND]--------------+            |
                   +------------+

--- IMU
                   +------------+
[5V]---------------+ IMU        |
[A0]---------------+-X          |
[A1]---------------+-Y          |
[A2]---------------+-Z          |
[GND]--------------+            |
                   +------------+
                   
--- Switches/Printer                   
                   +------------+
[5V]---------------+ SWITCH/PRT |
[ 8]---------------+ <switch>   |
[ 9]---------------+-<MAX232>---+--<printer>
[10]---------------+ <switch>   |
[GND]--------------+            |
                   +------------+

                      _ switch N/O
where -<switch> is --+ +--[GND]     

where -<MAX232>- is  
      +-----------------+
     ===                |
      |+   +---------+  +---+----[5V]
      |    |  MAX232 |  |   |+
+ +--------+[1]  [16]+--+  ===
 ===  +----+         +------+----[GND]
  +--------+         +-->----> to printer pin 2 (& GND to pin 5)
           +--  +    |              
           +--===    |
        +--+         +--<----< TX data from Arduno
        |  |         |
        |  |         |
        |  +---------+
       ===
        | +
       [GND]
All caps (===) are 1uF
[+ve] [-ve]
5V     GND
1      3
4      5
GND    6
2      5V

--- LCD
                                           +----------------+
                                           |  QC12864 LCD   |
                                      [GND]+20              |
                   +------------+   +-<pot>+19 Backlight    |
[5V]---------------+   74HC595  |   |       :               |
[ 5]---------------+-CLOCK    b1+---+  [5V]+15 PSB          |
[ 6]---------------+-LATCH    b2+----------+14 DB7          |
[ 7]---------------+-DATA     b3+----------+13 DB6          |
[GND]--------------+          b4+----------+12 DB5          |
                   |          b5+----------+11 DB4          |
                   |          b6+--------+ :                |
                   |          b7+--+     +-+ 6 E(SCLK)      |
                   +------------+  |  [GND]+ 5 R/W(SID)     |
                                   +-------+ 4 RS(CS)       |
                                      <pot>+ 3 Contrast     |
                                       [5V]+ 2              |
                                      [GND]+ 1              |
                                           +----------------+

*/
#endif
