#include <Arduino.h>
#include "Printer.h"
#include "PINS.h"

Printer printer;
SoftwareSerial serial(0xFF, PIN_PRT_TX);

Printer::Printer()
{
}

void Printer::Init()
{
  serial.begin(9600);
  for (int Clr = 0; Clr < 256; Clr++)
  {
    serial.write((byte)0x10);
  }
  serial.write((byte)0x1B);
  serial.write((byte)0x40);
  delay(500);
}
 
void Printer::Println(char* Str, bool BigBold)
{
  if (BigBold)
  {
    serial.write((byte)0x1B);
    serial.write((byte)0x21);
    serial.write((byte)B00111000);  // double width, height and emphasised
    serial.println(Str);
    serial.write((byte)0x1B);
    serial.write((byte)0x21);
    serial.write((byte)B00000000);
  }
  else
  {
    serial.println(Str);
  }
}

void Printer::StartPrint()
{
  serial.write((byte)0x10);
}

void Printer::PrintBand(byte* pBand, int Bytes)
{
  serial.write((byte)0x1B);
  serial.write((byte)0x2A);
  serial.write((byte)0x00); // 8, single
  serial.write((byte)lowByte(Bytes));
  serial.write((byte)highByte(Bytes));
  for (int Idx = 0; Idx < Bytes; Idx++)
  {
    serial.write((byte)pBand[Idx]);
  }
  // print and feed
  serial.write((byte)0x1B);
  serial.write((byte)0x64);
  serial.write((byte)0x00);
}

void Printer::PrintBandDouble(byte* pBand, int Bytes)
{
  serial.write((byte)0x10);
  for (int Pass = 0; Pass < 2; Pass++)
  {
    serial.write((byte)0x1B);
    serial.write((byte)0x2A);
    serial.write((byte)0x00); // 8, single
    serial.write((byte)lowByte(Bytes*2));
    serial.write((byte)highByte(Bytes*2));
    for (int Idx = 0; Idx < Bytes; Idx++)
    {
      byte Single = pBand[Idx];
      byte Double = 0;
      int ReadBit = (Pass == 0)?4:0;
      for (int WriteBit = 0; WriteBit < 8; WriteBit += 2)
      {
        // upper
        bitWrite(Double, WriteBit, bitRead(Single, ReadBit));
        bitWrite(Double, WriteBit + 1, bitRead(Single, ReadBit));
        ReadBit++;
      }
      serial.write((byte)Double);
      serial.write((byte)Double);
    }
    serial.write((byte)0x1B);
    serial.write((byte)0x64);
    serial.write((byte)0x00);
  }
}

void Printer::FeedLines(byte N)
{
  serial.write((byte)0x14);
  serial.write((byte)N);  // feed n lines
}

void Printer::EndPrint()
{
  FeedLines(8);
  serial.write((byte)0x19);  // cut
}
